use sea_battle_backend::data::{GameRules, PlayConfiguration};
use tui::buffer::Buffer;
use tui::layout::Rect;
use tui::widgets::{BorderType, Widget};

pub struct GameMapWidget<'a> {
    rules: &'a GameRules,
}

impl<'a> GameMapWidget<'a> {
    pub fn new(rules: &'a GameRules) -> Self {
        Self { rules }
    }
}

impl<'a> Widget for GameMapWidget<'a> {
    fn render(self, area: Rect, buf: &mut Buffer) {
        let alphabet = PlayConfiguration::default().ordinate_alphabet;

        let symbols = BorderType::line_symbols(BorderType::Plain);

        // Paint game grid
        for y in 0..(self.rules.map_height + 1) {
            for x in 0..(self.rules.map_width + 1) {
                let o_x = x as u16 * 2;
                let o_y = y as u16 * 2;
                buf.get_mut(o_x, o_y).set_symbol(match (x, y) {
                    (0, 0) => symbols.top_left,

                    (x, 0) if x == self.rules.map_width => symbols.top_right,
                    (0, y) if y == self.rules.map_height => symbols.bottom_left,

                    (0, _) => symbols.vertical_right,
                    (_, 0) => symbols.horizontal_down,

                    (x, y) if x == self.rules.map_width && y == self.rules.map_height => {
                        symbols.bottom_right
                    }

                    (x, _) if x == self.rules.map_width => symbols.vertical_left,
                    (_, y) if y == self.rules.map_height => symbols.horizontal_up,

                    _ => symbols.cross,
                });

                if x < self.rules.map_width {
                    buf.get_mut(o_x + 1, o_y).set_symbol(symbols.horizontal);
                }

                if y < self.rules.map_height {
                    buf.get_mut(o_x, o_y + 1).set_symbol(symbols.vertical);
                }

                if x < self.rules.map_width && y < self.rules.map_height {
                    buf.get_mut(o_x + 1, o_y + 1).set_char('.');
                }
            }
        }
    }
}
